/* ----------------------------------------------------------------------------
 *         ATMEL Microcontroller Software Support
 * ----------------------------------------------------------------------------
 * Copyright (c) 2009, Atmel Corporation
 *
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions are met:
 *
 * - Redistributions of source code must retain the above copyright notice,
 * this list of conditions and the disclaimer below.
 *
 * Atmel's name may not be used to endorse or promote products derived from
 * this software without specific prior written permission.
 *
 * DISCLAIMER: THIS SOFTWARE IS PROVIDED BY ATMEL "AS IS" AND ANY EXPRESS OR
 * IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
 * MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND NON-INFRINGEMENT ARE
 * DISCLAIMED. IN NO EVENT SHALL ATMEL BE LIABLE FOR ANY DIRECT, INDIRECT,
 * INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
 * LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA,
 * OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,
 * EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 * ----------------------------------------------------------------------------
 */


/*----------------------------------------------------------------------------
  Headers
----------------------------------------------------------------------------*/
#include "atmel_app.h"

/*----------------------------------------------------------------------------
 *        Local definitions
 *----------------------------------------------------------------------------*/

/** Reference voltage for ADC,in mv*/
#define VOLT_REF   (3300)

/* The maximal digital value*/
#define MAX_DIGITAL (4095)

/** channel connected to internal Temperature Sensor */
#define ADC_TEMP_SENSOR_CHANNEL     ADC_CHANNEL_15

/** channel connected to on-board potentiometer*/
#define ADC_POT_CHANNEL             ADC_CHANNEL_5

/* Qtouch */
#define GET_SENSOR_STATE(SENSOR_NUMBER) (qt_measure_data.qt_touch_status.sensor_states[(SENSOR_NUMBER/8)] & (1 << (SENSOR_NUMBER % 8)))
#define GET_ROTOR_SLIDER_POSITION(ROTOR_SLIDER_NUMBER) qt_measure_data.qt_touch_status.rotor_slider_values[ROTOR_SLIDER_NUMBER]


/*----------------------------------------------------------------------------
                                extern variables
----------------------------------------------------------------------------*/
/* This configuration data structure parameters if needs to be changed will be
   changed in the qt_set_parameters function */
extern qt_touch_lib_config_data_t qt_config_data;
/* measurement data */
extern qt_touch_lib_measure_data_t qt_measure_data;
/* Get sensor delta values */
extern int16_t qt_get_sensor_delta( uint8_t sensor);
/* Output can be observed in the watch window using this pointer */
qt_touch_lib_measure_data_t *pqt_measure_data = &qt_measure_data;

/* QDebug */
extern uint16_t timestamp1_hword;
extern uint16_t timestamp1_lword;
extern uint16_t timestamp2_hword;
extern uint16_t timestamp2_lword;
extern uint16_t timestamp3_hword;
extern uint16_t timestamp3_lword;

/*----------------------------------------------------------------------------
 *        Local variables
 *----------------------------------------------------------------------------*/
/** time stamp */
uint32_t time_stamp = 0;
/* Timer period in msec. */
uint16_t qt_measurement_period_msec = 25u;

/*status flags to indicate the re-burst for library*/
uint16_t status_flag = 0u;
uint16_t burst_flag = 0u;

uint8_t vld_pressed = 0;
uint8_t up_pressed = 0;
uint8_t dwn_pressed = 0;
uint8_t lft_pressed = 0;
uint8_t rgt_pressed = 0;

/*----------------------------------------------------------------------------
  static variables
----------------------------------------------------------------------------*/
/* Qtouch */
/* flag set by timer ISR when it's time to measure touch */
static volatile uint8_t time_to_measure_touch = 0u;

/* current time, set by timer ISR */
static volatile uint16_t current_time_ms_touch = 0u;



/*----------------------------------------------------------------------------
 *        Local functions
 *----------------------------------------------------------------------------*/
/* initialise host app, pins, watchdog, etc */
//static void init_system( void );

/* configure timer ISR to fire regularly */
static void init_timer_isr( void );

/*  Assign the parameters values to global configuration parameter structure    */
static void qt_set_parameters( void );

/* Configure the sensors */
static void config_sensors(void);


/**
 * \brief  Global ADC configuration
 */
void _AdcInit()
{
    /* Initialize ADC */
    ADC_Initialize( ADC, ID_ADC );

    /* Enable Freerun mode for start conversion erratum */
    ADC_SetTrigger( ADC, 0, 0 );
    ADC_SetFreeRunMode( ADC, 1 ) ;

    /*  startup = 8:    512 periods of ADCClock
     * for prescal = 4
     *     prescal: ADCClock = MCK / ( (PRESCAL+1) * 2 ) => 64MHz / ((4+1)*2) = 6.4MHz
     *     ADC clock = 6.4 MHz
     */
    ADC_SetClock( ADC, 4, BOARD_MCK );
    ADC_SetTiming( ADC, 8, 1, 0, 3 );

    ADC_CheckConfiguration( ADC, BOARD_MCK );

}

/**
 * \brief  Init Internal temperature sensor on ADC channel 15
 */

void _AdcTempSensorInit( void )
{

    /*Enable ADC channel for the temperature sensor*/
    ADC_EnableChannel( ADC,  ADC_TEMP_SENSOR_CHANNEL ) ;

    /*Enable the temperature sensor*/
    ADC_EnableTS( ADC ) ;

    /*Enable the temperature sensor*/
    ADC_EnableIt(ADC, 0x1 << ADC_TEMP_SENSOR_CHANNEL ) ;

}

/**
 * \brief  Init Internal on-board potentiometer
 */

void _AdcPotentiometerInit( void )
{

    /*Enable  channel for potentiometer*/
    ADC_EnableChannel( ADC, ADC_POT_CHANNEL ) ;
    ADC_EnableIt(ADC, 0x1 << ADC_CHANNEL_5 ) ;

}


/**
 * \brief  Return converted value from temp sensor. Raw digital value
 */

uint16_t _AdcReadTempSensor( void )
{
    unsigned int status ;

    status = ADC_GetStatus(ADC);

    while ( (status & ADC_ISR_EOC15)!=ADC_ISR_EOC15)
    {
      status = ADC_GetStatus(ADC);
    }

    return ( ADC_GetConvertedData( ADC, ADC_CHANNEL_15) ) ;
}

/**
 * \brief  Return converted value from potentiometer. Raw digital value
 */
uint16_t _AdcReadAnalogInput( void )
{
    unsigned int status ;

    status = ADC_GetStatus(ADC);

    while ( (status & ADC_ISR_EOC5)!=ADC_ISR_EOC5)
    {
      status = ADC_GetStatus(ADC);
    }

    return ( ADC_GetConvertedData( ADC, ADC_CHANNEL_5) ) ;
}


/**
 *  \brief Configure LEDs
 *
 *  Configures LEDs \#1 and \#2 (cleared by default).
 */
void _ConfigureLeds( void )
{
    LED_Configure( 0 ) ;
    LED_Configure( 1 ) ;
}

/**
 * \brief  Toggle LED
 */

void _IoLed( uint8_t ubIdx1, uint8_t ubIdx2 )
{
    // Manage BLUE LED
    if ( ubIdx1 )
    {
      LED_Set( 0 );
    }
    else
    {
      LED_Clear( 0 );
    }

    // Manage GREEN LED
    if ( ubIdx2 )
    {
      LED_Set( 1 );
    }
    else
    {
      LED_Clear( 1 );
    }

}


/**
 * \brief  Qtouch Initialization
 */

void _QtouchSliderInit ( void )
{
    /* Qtouch Hardware Init */
    /* reset touch sensing */
    qt_reset_sensing();

    /* Configure the Sensors as keys or Keys With Rotor/Sliders in this function */
    config_sensors();

    /* initialise touch sensing */
    qt_init_sensing();

    /* Set the parameters like recalibration threshold, Max_On_Duration etc in this function by the user */
    qt_set_parameters( );

    /* configure timer ISR to fire regularly */
    init_timer_isr();

    /* Address to pass address of user functions */
    /* This function is called after the library has made capacitive measurements,
    * but before it has processed them. The user can use this hook to apply filter
    * functions to the measured signal values.(Possibly to fix sensor layout faults) */
    qt_filter_callback = 0;

}

/**
 * \brief  Return the Slider position. 8-bit value
 */

uint8_t _ReadQtouchSlider ( void )
{


    static uint8_t old_position = 0;
    if( time_to_measure_touch )
    {
      /* clear flag: it's time to measure touch */
      time_to_measure_touch = 0u;
      do
      {
        /*  one time measure touch sensors    */
        status_flag = qt_measure_sensors( current_time_ms_touch );
        burst_flag = status_flag & QTLIB_BURST_AGAIN;
        /*Time critical host application code goes here*/
      } while (burst_flag);
    }

      /*  Time Non-critical host application code goes here  */
    if( GET_ROTOR_SLIDER_POSITION(0) != old_position )
    {
      old_position = GET_ROTOR_SLIDER_POSITION(0);
    }
    return ( old_position );
}



void qt_set_parameters( void )
{
    /*  This will be modified by the user to different values   */
    qt_config_data.qt_di              = DEF_QT_DI;
    qt_config_data.qt_neg_drift_rate  = DEF_QT_NEG_DRIFT_RATE;
    qt_config_data.qt_pos_drift_rate  = DEF_QT_POS_DRIFT_RATE;
    qt_config_data.qt_max_on_duration = DEF_QT_MAX_ON_DURATION;
    qt_config_data.qt_drift_hold_time = DEF_QT_DRIFT_HOLD_TIME;
    qt_config_data.qt_recal_threshold = DEF_QT_RECAL_THRESHOLD;
    qt_config_data.qt_pos_recal_delay = DEF_QT_POS_RECAL_DELAY;
}
/**
 *  \brief Configure the sensors
 *
 *      PIOC     0  1  chanel 0
 *      PIOC     2  3  chanel 1
 *      PIOC     4  5  chanel 2
 *      PIOC     6  7  chanel 3
 *      PIOC     8  9  chanel 4
 *      PIOC    10 11  chanel 5
 *      PIOC    12 13  chanel 6
 *      PIOC    14 15  chanel 7
 *      PIOC    16 17  chanel 8
 *      PIOC    18 19  chanel 9
 *      PIOC    20 21  chanel 10
 *      PIOA     0  1  chanel 16   : slider
 *      PIOA     2  3  chanel 17   : slider
 *      PIOA     4  5  chanel 18   : slider
 *      PIOA     6  7  chanel 19
 *      PIOA     8  9  chanel 20
 *      PIOA    10 11  chanel 21
 *      PIOA    12 13  chanel 22
 *      PIOA    14 15  chanel 23
 *      PIOA    16 17  chanel 24
 *      PIOA    18 19  chanel 25
 *      PIOA    20 21  chanel 26
 *      PIOA    22 23  chanel 27
 *      PIOA    24 25  chanel 28
 *      PIOA    26 27  chanel 29
 *      PIOA    28 29  chanel 30
 *      PIOA    30 31  chanel 31
 *
 */
void config_sensors(void)
{
    PMC_EnablePeripheral(ID_PIOA);
    PMC_EnablePeripheral(ID_PIOC);

    /* Slider: PA0, PA1, PA2, PA3, PA4, PA5 */
    qt_enable_slider( CHANNEL_16, CHANNEL_18, AKS_GROUP_1, 16u, HYST_6_25, RES_8_BIT, 0u  );
}



/**
 *  \brief  Return Temperature. No formating
 *
 *  Set systick event flag
 */
uint16_t Read_Temperature(void)
{
  return ( _AdcReadTempSensor());
}


/**
 *  \brief  Handler for Sytem Tick interrupt.
 *
 *  Set systick event flag
 */
void SysTick_Handler(void)
{
    /* set flag: it's time to measure touch */
    time_to_measure_touch = 1u;

    /* update the current time */
    current_time_ms_touch += qt_measurement_period_msec;
}

/**
 *  \brief  configure timer ISR to fire regularly
 *
 *  Configure Timer Counter 0 to generate an interrupt every 25ms.
 */
void init_timer_isr( void )
{
    SysTick_Config(BOARD_MCK / (1000 / qt_measurement_period_msec));
}
